/*
 * simco_b.h
 *
 * Copyright (C) 2005, Ranch Networks, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * Author : Srivatsa Chivukula <srivatsa@ranchnetworks.com>
 *
 */

#ifndef SIMCO_B_H
#define SIMCO_B_H


extern int b_mcAgentTCPSessionEstablish(MCAgentCon *con);
extern int b_mcAgentTCPSessionAuthenticate(MCAgentCon *con, char *username, char *password);
extern int b_mcAgentTCPSessionTerminate(MCAgentCon *con);
extern int b_mcAgentTCPPing(MCAgentCon *con);
extern int b_mcAgentTCPPolicyReserveRule(MCAgentCon *con, simcoPacket_t *sp);
extern int b_mcAgentTCPPolicyEnableRule(MCAgentCon *con, simcoPacket_t *sp);
extern int b_mcAgentTCPPERAfterReservation(MCAgentCon *con, simcoPacket_t *sp);
extern int b_mcAgentTCPPBRAfterReservation(MCAgentCon *con, simcoPacket_t *sp);
extern int b_mcAgentTCPPolicyRuleLifetimeChange(MCAgentCon *con, simcoPacket_t *sp);
extern int b_mcAgentTCPPolicyRuleStatus(MCAgentCon *con, simcoPacket_t *sp);
extern int b_mcAgentTCPPolicyRuleList(MCAgentCon *con, int **rules, int *numrules);
extern int b_mcAgentTCPNotificationRead(MCAgentCon *con, simcoPacket_t *np);


#endif /* SIMCO_B_H */
