/*
 * sysError.h
 *
 * Copyright (C) 2005, Ranch Networks, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * Author : Srivatsa Chivukula <srivatsa@ranchnetworks.com>
 *
 */

#ifndef _SYS_ERROR_H
#define _SYS_ERROR_H

#define SYS_ERROR_BASE  0

#define SYS_SUCCESS		 0	/* operation sucessful */
#define SYS_FAILURE		 1	/* operation failed */
#define SYS_CFAILURE		 2
#define SYS_EINVPARM		 3
#define SYS_ENOPERM		 4
#define SYS_EBUSY		 5
#define SYS_EINUSE		 6
#define SYS_ENOMEM		 7
#define SYS_ENRESOURCE		 8
#define SYS_EINVRANGE		 9
#define SYS_ETIMEOUT		10
#define SYS_EWBLOCK		11
#define SYS_EUNAVAIL		12
#define SYS_EEXISTS		13
#define SYS_ENFOUND		14
#define SYS_ESTRANS		15
#define SYS_EINIT		16
#define SYS_ETOOBIG		17
#define SYS_ETOOSMALL		18
#define SYS_EINVOP		19
#define SYS_EMEMCRPTN		20
#define SYS_EINVTCTX		21
#define SYS_ENOWNER		22
#define SYS_EINVPTR		23
#define SYS_ENIMPL		24
#define SYS_EMAXNO		25
#define SYS_EINACTIVE		26
#define SYS_ENSUPPORT		27

#define SYS_PARTIAL_FAILURE              0xFFFFFFFE
#define SYS_SUCCESS_DATA_CHANGED         0xFFFFFFFD
#define SYS_PARTIAL_FAILURE_DATA_CHANGED 0xFFFFFFFC

#endif /* _SYS_ERROR_H */
