/*
 * simco_nb_ext.h
 *
 * Copyright (C) 2005, Ranch Networks, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * Author : Srivatsa Chivukula <srivatsa@ranchnetworks.com>
 *
 */

#ifndef SIMCO_NB_EXT_H
#define SIMCO_NB_EXT_H

#include "simco_core_ext.h"
#include "simco_nb.h"

extern int nb_init_ext(void);
extern void reader_thread_parsing_contd(int index, int i, char *resp_buffer);

extern int nb_mcAgentTCPFirewallRuleCreate(MCAgentCon *con, const simcoExtPacket_t *sep);
extern int nb_mcAgentTCPFirewallRuleDelete(MCAgentCon *con, const simcoExtPacket_t *sep);
extern int nb_mcAgentTCPFirewallRuleStatus(MCAgentCon *con, const simcoExtPacket_t *sep);
extern int nb_mcAgentTCPFirewallRuleList(MCAgentCon *con);

#endif /* SIMCO_NB_EXT_H */


